import java.text.SimpleDateFormat

import org.apache.commons.codec.binary.Base64
import org.apache.commons.io.IOUtils
import org.apache.log4j.Logger

import com.atlassian.jira.component.ComponentAccessor

import groovy.json.JsonSlurper

xray = new XrayTool(restUserName: restUserName, restPassword: restPassword, baseUrl: baseurl)

/**
 * Velocity tool to get Xray test data using the Xray REST API.
 */
class XrayTool {
	def log = Logger.getLogger(this.getClass())

	def userManager = ComponentAccessor.userManager

	/* Xray REST API request parameters. */
	def baseUrl
	def restUserName
	def restPassword

	/**
	 * Some Xray REST end-points return dates in this hardwired format.
	 */
	def static xrayDateFormat = new SimpleDateFormat("yyyy-MM-dd'T'hh:mm:ssXXX", ComponentAccessor.jiraAuthenticationContext.locale)
	def jsonSlurper = new JsonSlurper()

	/**
	 * Returns the Xray test executions for the passed test plan type issue (as Groovy objects slurped from <code>https://confluence.xpand-it.com/display/public/XRAY/Test+Plans+-+REST<code>)
	 */
	def getTestExecutionsByTestPlan(def issue) {
		def issueKey = issue.key
		log.debug("Downloading Xray test executions for issue #${issueKey}")
		def response = callRestApi("${baseUrl}/rest/raven/1.0/api/testplan/${issueKey}/testexecution")
		if (!response instanceof Throwable) {
			log.debug("${response.size()} Xray test executions found for issue #${issueKey}")
		}
		return response
	}

	/**
	 * Returns the Xray test runs for the passed test type issue (as Groovy objects slurped from <code>https://confluence.xpand-it.com/display/public/XRAY/Tests+-+REST#Tests-REST-ExportingTestRuns<code>)
	 */
	def getTestRunsByTest(def issue) {
		def issueKey = issue.key
		log.debug("Downloading Xray test runs for issue #${issueKey}")
		def response = callRestApi("${baseUrl}/rest/raven/1.0/api/test/${issueKey}/testruns")
		if (!response instanceof Throwable) {
			log.debug("${response.size()} Xray test runs found for issue #${issueKey}")
		}
		return response
	}

	def callRestApi(def url) {
		def connection
		def inputStream

		try {
			log.debug("Xray REST API URL: ${url}")
			connection = new URL(url).openConnection()
			def authValue = "${restUserName}:${restPassword}"
			connection.setRequestProperty("Authorization", "Basic " + Base64.encodeBase64String(authValue.getBytes("ISO-8859-1")))
			inputStream = connection.getInputStream()
			def json = IOUtils.toString(inputStream, "UTF-8")
			log.trace("Xray JSON for ${url}:\n${json}")

			def result = jsonSlurper.parseText(json)

			return result
		} catch (Throwable ex) {
			log.error("Failed to download Xray data", ex)
			return ex
		} finally {
			IOUtils.closeQuietly(inputStream)
		}
	}

	def getDate(def dateString) {
		dateString ? xrayDateFormat.parse(dateString) : null
	}

	def getDisplayName(def userKey) {
		userKey ? userManager.getUserByKey(userKey).displayName : null
	}
}
